/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#include "service_tunein.h"
#include "networkaccess.h"
#include "button_item.h"
#include "utilities.h"
#include "debug.h"

/* 
API : 
 Local Radio -> http://opml.radiotime.com/Browse.ashx?c=local
 Talk        -> http://opml.radiotime.com/Browse.ashx?c=talk
 Sports      -> http://opml.radiotime.com/Browse.ashx?c=sports
 Music       -> http://opml.radiotime.com/Browse.ashx?c=music
 By Location -> http://opml.radiotime.com/Browse.ashx?id=r0
*/
        
/*
********************************************************************************
*                                                                              *
*  Global                                                                      *
*                                                                              *
********************************************************************************
*/
namespace TUNEIN {
  namespace Global {
        QString partner_id;
        QString username;
        QString password;
  }
}




/*
********************************************************************************
*                                                                              *
*    Class TuneIn                                                              *
*                                                                              *
********************************************************************************
*/
TuneIn::TuneIn() : Service("Tunein", SERVICE::TUNEIN, true)
{
    Debug::debug() << "TuneIn -> create";
    
    /* restore settings */
//     QSettings settings(UTIL::CONFIGFILE,QSettings::IniFormat,this);
//     settings.beginGroup("TuneIn");
    
    TUNEIN::Global::partner_id   = "yvcOjvJP";
//     TuneIn::Global::username     = settings.value("username", QString()).toString();
//     TuneIn::Global::password     = settings.value("pass", QString()).toString();
//     settings.endGroup();
    
    /* root link */
    m_root_link = MEDIA::LinkPtr(new MEDIA::Link());
    m_root_link->url  = QString("http://opml.radiotime.com/Browse.ashx");
    m_root_link->name = QString("tunein directory");
    m_root_link->state = int (SERVICE::NO_DATA);
   
    m_active_link = m_root_link;
    set_state(SERVICE::NO_DATA);    
}

void TuneIn::reload()
{
    /* register update */    
    m_active_link->state = int(SERVICE::NO_DATA);
    set_state(SERVICE::NO_DATA);    
    
    emit stateChanged();
}

void TuneIn::load()
{
    Debug::debug() << "TuneIn::load";
    if(state() == SERVICE::DOWNLOADING)
      return;
    
    m_active_link->state = int(SERVICE::DOWNLOADING);
    set_state(SERVICE::DOWNLOADING);
    
    emit stateChanged();

    browseLink(m_active_link);
}

QList<MEDIA::LinkPtr> TuneIn::links() const
{
    Debug::debug() << "TuneIn::links";

    QList<MEDIA::LinkPtr> links;

    foreach(MEDIA::MediaPtr media, m_root_link->children()) {
      if(media->type() == TYPE_LINK) {
       MEDIA::LinkPtr link = MEDIA::LinkPtr::staticCast(media);
       links << link;
      }
    }
    
    if(m_active_link != m_root_link) 
    {      
      foreach(MEDIA::MediaPtr media, m_active_link->children()) {
        if(media->type() == TYPE_LINK) {
          MEDIA::LinkPtr link = MEDIA::LinkPtr::staticCast(media);
          links << link;
        }
      }
    }
    return links;
}

QList<MEDIA::TrackPtr> TuneIn::streams()
{
    QList<MEDIA::TrackPtr> streams;
  
    foreach(MEDIA::MediaPtr media, m_active_link->children()) {
      if(media->type() == TYPE_STREAM) {
        MEDIA::TrackPtr stream = MEDIA::TrackPtr::staticCast(media);
        streams << stream;
      }
    }
      
    return streams;
}


void TuneIn::browseLink(MEDIA::LinkPtr link)
{
    //http://opml.radiotime.com/Browse.ashx?c=local&partnerId=<partnerid>&serial=<serial>
    Debug::debug() << "TuneIn::browseLink : " << link->url;
    QNetworkRequest request;
    request.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");

    QUrl url;
    url.setUrl(link->url);
    if(!url.hasQueryItem("partnerId"))
      url.addQueryItem("partnerId", TUNEIN::Global::partner_id);    
    if(!url.hasQueryItem("render"))
      url.addQueryItem("render", "xml");
    
    QObject *reply = HTTP()->get(url);
    m_requests[reply] = link;    
    connect(reply, SIGNAL(data(QByteArray)), SLOT(slotBrowseLinkDone(QByteArray)));
    connect(reply, SIGNAL(error(QNetworkReply*)), this, SLOT(slotBrowseLinkError()));
} 

void TuneIn::slotBrowseLinkDone(QByteArray bytes)
{
    Debug::debug() << "TuneIn::slotBrowseLinkDone";
    QObject* reply = qobject_cast<QObject*>(sender());
    if (!reply || !m_requests.contains(reply))
      return;
    
    MEDIA::LinkPtr link = m_requests.take(reply);    
    
    /* parse response */
    QXmlStreamReader xml(bytes);

    while(!xml.atEnd() && !xml.hasError()) {
      
      xml.readNext();
      if (xml.isStartElement()) {
          if(xml.name() == "outline")
          {
            if(xml.attributes().value("type").toString() == "audio")
            {
              //Debug::debug() << "TuneIn::stream found";
              MEDIA::TrackPtr stream = MEDIA::TrackPtr::staticCast( link->addChildren(TYPE_TRACK) );
              stream->setType(TYPE_STREAM);
              stream->name = xml.attributes().value("text").toString();
              stream->url  = xml.attributes().value("URL").toString();

//               if(xml.attributes().hasAttribute("image"))
//                 stream->image_url  = xml.attributes().value("image").toString();

              stream->categorie = link->name;
              stream->setParent(link);
            }
            else if (xml.attributes().value("type").toString() == "link")
            {
              //Debug::debug() << "TuneIn::link found";
              MEDIA::LinkPtr link2 = MEDIA::LinkPtr::staticCast( link->addChildren(TYPE_LINK) );
              link2->setType(TYPE_LINK);
              link2->name  = xml.attributes().value("text").toString();
              link2->url   = xml.attributes().value("URL").toString();
              link2->state = int(SERVICE::NO_DATA);
              link2->categorie = link->name;
              link2->setParent(link);
            }
          }
      }
    }    
    
    
    /* register update */    
    link->state = int(SERVICE::DATA_OK);
    set_state(SERVICE::DATA_OK);    
    
    emit stateChanged();
}  

void TuneIn::slotBrowseLinkError()
{
    Debug::debug() << "TuneIn::slotBrowseLinkError";
   
    QObject* reply = qobject_cast<QObject*>(sender());
    if (!reply || !m_requests.contains(reply))
      return;
    
    MEDIA::LinkPtr link = m_requests.take(reply);    
    
    link->state = int(SERVICE::ERROR);
    set_state(SERVICE::ERROR);    
    emit stateChanged();
}  

void TuneIn::slot_activate_link()
{
    ButtonItem* button = qobject_cast<ButtonItem*>(sender());
    if (!button) return;  
  
    MEDIA::LinkPtr link = qvariant_cast<MEDIA::LinkPtr>( button->data() );

    if(link) {
      m_active_link = link;
      set_state(SERVICE::State(m_active_link->state));
      emit stateChanged();
    }
}

// void TuneIn::signIn()
// {
//     /* request :
//      https://opml.radiotime.com/Account.ashx?c=auth&partnerId=<partnerid>&username=<username>&password=<password>
//     */
//      
//     Debug::debug() << "TuneIn::SignIn";
//     QNetworkRequest request;
//     request.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded");
// 
//     QUrl url;
//     url.setUrl("https://opml.radiotime.com/Account.ashx");
//     url.addQueryItem("c", "auth");
//     url.addQueryItem("partnerId", TuneIn::Global::partner_id);
//     url.addQueryItem("username",TuneIn::Global::username);
//     url.addQueryItem("password", TuneIn::Global::password);
//     request.setUrl(url.toString());
// 
//     QNetworkAccessManager *nam = new QNetworkAccessManager();
//     connect(nam, SIGNAL(finished(QNetworkReply*)),this, SLOT(slotSignInFinished(QNetworkReply*)));
//     nam->get(request);
// }
// 
// void TuneIn::slotSignInFinished()
// {
//     int status = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
//     if (status != 200)
//     {
//         Debug::warning() << "TuneIn::SignIn failed";
//         isAuthenticated = false;
//         return;
//     }  
//     
//     isAuthenticated = true;
//     Debug::debug() << "TuneIn::SignIn OK";
// }
