/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/
#ifndef _MAINWINDOW_H_
#define _MAINWINDOW_H_

#include "threadmanager.h"

// qt
#include <QMainWindow>
#include <QSystemTrayIcon>



class CommandlineOptions;

// player engine
class EnginePlayer;

// widgets
class PlaylistView;
class PlaylistWidget;
class NowPlayingView;
class GraphicViewWidget;
class CentralWidget;
class MenuView;
class MenuModel;
class MinimalWidget;
class StatusWidget;
class CentralToolBar;

// Model
class PlaylistModel;
class CollectionModel;
class PlsFileModel;
class StreamModel;
class HistoModel;

// Views
class CollectionScene;
class StreamScene;
class SettingsScene;
// Core
class ScrobblerManager;
class ThreadManager;
class HistoManager;
class DatabaseManager;
class VirtualPlayqueue;
class GlobalShortcuts;

// Dbus & Mpris
class DbusNotification;
class MprisManager;
/*
********************************************************************************
*                                                                              *
*    Class MainWindow                                                          *
*                                                                              *
********************************************************************************
*/
class MainWindow : public QMainWindow
{
Q_OBJECT
    static MainWindow         *INSTANCE;

  public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();
    static MainWindow* instance() { return INSTANCE; }

    void commandlineOptionsHandle(const CommandlineOptions& options);
  private:
    /* Mainwindow        */
    void createGui();
    void createActions();
    void connectSlots();

    /* Settings          */
    void reloadSystraySettings();
    void savePlayingTrack();
    void restorePlayingTrack();

    /* Database Method   */
    void removeDatabase();
    void createDatabase();
    void rebuildDatabase();

  private slots:
    /* Mainwindow        */
    void slot_systray_clicked(QSystemTrayIcon::ActivationReason);
    void slot_widget_mode_switch();
    void slot_commandline_received(const QByteArray& serialized_options);
    void slot_on_settings_saved();
    void slot_on_aboutYarock();
    void slot_on_aboutQt();
    void slot_on_yarock_quit();

    /* Playqueue        */
    void slot_playqueue_add();
    void slot_playqueue_clear();
    void slot_playqueue_save();
    void slot_playqueue_save_auto();
    void slot_playqueue_sort();

    /*  Player           */
    void slot_player_enqueue_next();
    void slot_player_on_state_change();
    void slot_play_from_playqueue();
    void slot_play_from_collection();
    void slot_player_on_track_change (const QString& file);
    void slot_stop_after_media_triggered();

    void playOrPause();
    void stopPlayer();
    void playNext();
    void playPrev();

    /* Database slots    */
    void slot_database_start();
    void slot_database_dialog();

    /* Data models & views  */
    void slot_views_update(E_MODEL_TYPE model_updated);

    /* Last FM           */
    void slot_LastFmLove();

    /* Playlist editor   */
    void slot_open_playlist_editor();
    void slot_delete_playlist_editor();

    /* Jump to item slot */
    void slot_jump_to_now_playing();
    void set_enable_jump_to(bool b);

    /* Equalizer         */
    void slot_eq_openDialog();
    void slot_eq_enableChange(bool eqActivated);
    void slot_eq_paramChange(int, QList<int>);

  protected:
    void closeEvent(QCloseEvent *event);

  private:
    bool                 is_first_start;

    enum E_PLAYING_FROM   {FromCollection, FromPlayQueue};

    E_PLAYING_FROM        _playRequestFrom;

    EnginePlayer          *_player;

    StatusWidget          *_statusWidget;

    CentralToolBar        *_centralToolBar;
    CentralWidget         *_centralWidget;
    MenuView              *_menuView;
    MenuModel             *_menuModel;

    PlaylistView          *_playlistView;
    
    PlaylistModel         *m_playqueue;

    PlaylistWidget        *_playlistwidget;
    NowPlayingView        *_nowplayingview;

    CollectionModel       *_collectionModel;
    PlsFileModel          *_plsFileModel;
    HistoModel            *_histoModel;
    CollectionScene       *_collectionScene;
    GraphicViewWidget     *_graphicViewWidget;

    VirtualPlayqueue      *m_virtual_queue;

    StreamModel           *_streamModel;
    StreamScene           *_streamScene;
    SettingsScene         *m_settingsScene;

    ThreadManager         *m_thread_manager;

    DatabaseManager       *m_dbManager;

    QIcon                  *m_appIcon;
    QSystemTrayIcon        *m_trayIcon;
    bool                    m_canClose;

    MinimalWidget          *m_minimalwidget;
    HistoManager           *m_histoManager;

    ScrobblerManager       *m_scrobblerManager;

    GlobalShortcuts        *m_globalShortcuts;
    DbusNotification       *m_dbus_notifier;
    MprisManager           *m_mpris_manager;

signals:
    void playlistFinished();
};

#endif // _MAINWINDOW_H_
